using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Abdomen : FlexRegion
    {
        private RBDynamics rbDynamicAlt;
        
        public override void Init(int id, string[] rbNames)
        {
            name = "Abdomen";
            base.Init(id, rbNames);
            
            rbDynamics = new RBDynamics[1];
            var rb = atom.forceReceivers.First(x => x.name == rbNames[0])
                .GetComponent<Rigidbody>();
            rbDynamics[0] = rb.AddRbDynamic();
            
            rb = atom.forceReceivers.First(x => x.name == rbNames[1])
                .GetComponent<Rigidbody>();
            rbDynamicAlt = rb.AddRbDynamic();
        }

        public override void OnDestroy()
        {
            base.OnDestroy();
            Destroy(rbDynamicAlt);
        }
        
        protected override void GetMorphs()
        {
            for (int i = 0; i < 4; i++)
            {
                List<DAZMorph> sideMorphs = new List<DAZMorph>();
                var morph = FlexIt.GetMorphByUid($"FI_{name}{i}");
                if (morph == null) continue;
                morph.jsonFloat.constrained = false;
                sideMorphs.Add(morph);
                morphs.Add(sideMorphs);
                morphBuffers.Add(sideMorphs.Select(x => new MorphBuffer(x)).ToList());
            }
        }
        
        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            dynamicD = new Vector4(.35f, .125f, .5f, .5f);
            sRatioD.x = .5f;
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }
        
        protected override float GetFlex(int direction, int side)
        {
            float flex;
            if (direction < 2)
            {
                flex = rbDynamics[side].rigidbody.transform.forward.y;
            }
            else
            {
                flex = rbDynamics[side].rigidbody.transform.right.y;
            }

            return flex;
        }

        protected override bool Flexed(float flex, int direction, int side)
        {
            return direction == 0 || direction == 2? flex > 0f : flex < 0f;
        }
        
        protected override float GetAcceleration(int direction, int side)
        {
            float val;
            if (direction < 2) val = rbDynamics[side].angularAcceleration.x - .75f*rbDynamicAlt.angularAcceleration.x;
            else val = rbDynamics[side].angularAcceleration.z - .75f*rbDynamicAlt.angularAcceleration.z;
            val *= amount[direction];
            return val;
        }
    }
}