using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Ankle : FlexRegion
    {
        public override void Init(int id, string[] rbNames)
        {
            name = "Ankle";
            base.Init(id, rbNames);
            
            rbDynamics = new RBDynamics[2];
            for (int i = 0; i < 2; i++)
            {
                string side = sides[i];
                var rb = atom.forceReceivers.First(x => x.name == side.ToLower() + rbNames[0])
                    .GetComponent<Rigidbody>();
                rbDynamics[i] = rb.AddRbDynamic(i);
                SetupSupportTrackers(rb);
            }
        }

        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            amountD.x = 1.5f;
            amountD.y = 2f;
            dynamicD.x = .25f;
            dynamicD.y = .2f;
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }

        protected override void SetupSupportTrackers(Rigidbody rb)
        {
            // $"{rb.name} AnkleSetupSupportTrackers".Print();
            List<FlexCollisionTrigger> triggers = new List<FlexCollisionTrigger>();
            string side = rb.name.Substring(0, 1);
            if (side == "l")
            {
                lHeelTrigger = rb.transform.Find("lHeelColliderBase/lHeelColTOffset/lHeelColROffset/lHeelCollider/lHeelColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
                lToeTrigger = rb.transform.Find("lToe/lToeColliderBase/lToeColTOffset/lToeColROffset/lToeCollider/lToeColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
                if(lToeTrigger == null)
                    lToeTrigger = rb.transform.Find("lToeCollider/lToeColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
            }
            else
            {
                rHeelTrigger = rb.transform.Find("rHeelColliderBase/rHeelColTOffset/rHeelColROffset/rHeelCollider/rHeelColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
                rToeTrigger = rb.transform.Find("rToe/rToeColliderBase/rToeColTOffset/rToeColROffset/rToeCollider/rToeColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
                if(rToeTrigger == null)
                    rToeTrigger = rb.transform.Find("rToeCollider/rToeColliderTrigger")?.gameObject.AddComponent<HeelTrigger>();
                // rToeTrigger.NullCheck();
            }
            triggers.Add(rb.gameObject.AddComponent<FlexCollisionTrigger>());
            var toe = atom.rigidbodies.First(x => x.name == side + "Toe");
            var bigToe = atom.rigidbodies.First(x => x.name == side + "BigToe");
            var smallToe1 = atom.rigidbodies.First(x => x.name == side + "SmallToe1");
            triggers.Add(bigToe.gameObject.AddComponent<FlexCollisionTrigger>());
            triggers.Add(toe.gameObject.AddComponent<FlexCollisionTrigger>());
            var tracker = new FlexRegionSupportTracker(this, rb, triggers, smallToe1);
            supportTrackers.Add(tracker);
        }
        
        protected override float GetFlex(int direction, int side)
        {
            return FlexIt.MapRange(rbDynamics[side].transform.localRotation.x,
                .25f, -.15f, .7f, 1f, -1f);
        }
    }
}