using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Hand : FlexRegion
    {
        private List<Finger> fingers = new List<Finger>();
        
        public override void Init(int id, string[] rbNames)
        {
            name = "Hand";
            for (int i = 0; i < 4; i++)
            {
                fingers.Add(FlexIt.singleton.gameObject.AddComponent<Finger>());
                fingers[i].Init(id, i);
            }
            enabledJ.setCallbackFunction += val => enabled = val;
            dynamicStrengthJ.setCallbackFunction += val => fingers.ForEach(x => x.OnDynamicStrengthChanged(val));
            minAccelJ.setCallbackFunction += val => fingers.ForEach(x => x.minAccel = val);
            sRatioJ.setCallbackFunction += val => fingers.ForEach(x => x.OnSRatioChanged(val));
            amountJ.setCallbackFunction += val => fingers.ForEach(x => x.OnAmountChanged(val));
            
            SetupDefaults();
        }
        
        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            dynamicD.x = .5f;
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }
        
        public override void OnEnable()
        {
            foreach (var finger in fingers)
            {
                finger.enabled = true;
            }
        }

        public override void OnDisable()
        {
            foreach (var finger in fingers)
            {
                finger.enabled = false;
            }
        }
        
        public override void Reset()
        {
            foreach (var finger in fingers)
            {
                finger.Reset();
            }
        }

        public override void FixedUpdate()
        {
        }
    }
}