using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Knee : FlexRegion
    {
        public override void Init(int id, string[] rbNames)
        {
            name = "Knee";
            base.Init(id, rbNames);
            
            rbDynamics = new RBDynamics[2];
            for (int i = 0; i < 2; i++)
            {
                string side = sides[i];
                var rb = FlexIt.singleton.containingAtom.forceReceivers.First(x => x.name == side.ToLower() + rbNames[0])
                    .GetComponent<Rigidbody>();
                rbDynamics[i] = rb.AddRbDynamic(i);
                SetupSupportTrackers(rb);
            }
            SetupDefaults();
        }
        
        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            sRatioD.x = sRatioD.y =.75f;
            dynamicD.y = .75f;
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }
        
        protected override void SetupSupportTrackers(Rigidbody rb)
        {
            var triggers = new List<FlexCollisionTrigger>();
            var colliders = rb.transform.GetChild(3).transform.GetChildren().Select(x => 
                x.GetChild(0).GetComponent<Collider>()).ToList();
            colliders.Add(rb.GetComponentInChildren<Collider>());
            foreach (var collider in colliders)
            {
                var trigger = collider.gameObject.AddComponent<FlexCollisionTrigger>();
                triggers.Add(trigger);
            }
            if (triggers.Count > 0)
            {
                supportTrackers.Add(new FlexRegionSupportTracker(this, rb, triggers));
            }
        }
    }
}