using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Neck : FlexRegion
    {
        public override void Init(int id, string[] rbNames)
        {
            name = "Neck";
            base.Init(id, rbNames);
            
            rbDynamics = new RBDynamics[1];
            var rb = atom.forceReceivers.First(x => x.name == rbNames[0])
                .GetComponent<Rigidbody>();
            rbDynamics[0] = rb.AddRbDynamic();
        }
        
        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            amountD = new Vector4(2f, 1.5f, 1f, 1f);
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }
        
        protected override void GetMorphs()
        {
            for (int i = 0; i < 4; i++)
            {
                List<DAZMorph> sideMorphs = new List<DAZMorph>();
                var morph = FlexIt.GetMorphByUid($"FI_{name}{i}");
                if (morph == null) continue;
                morph.jsonFloat.constrained = false;
                sideMorphs.Add(morph);
                morphs.Add(sideMorphs);
                morphBuffers.Add(sideMorphs.Select(x => new MorphBuffer(x)).ToList());
            }
        }
        
        protected override float GetFlex(int direction, int side)
        {
            // "GetFlex".Print();
            // (rbDynamics[side].rigidbody.transform.localRotation.y).Print();
            if (direction < 2)
            {
                return rbDynamics[side].transform.forward.y;
            }
            return rbDynamics[side].transform.localRotation.y*10f;
        }

        protected override bool Flexed(float flex, int direction, int side)
        {
            if (direction < 2) return direction == 1? flex > 0f : flex < 0f;
            return direction == 2? flex > 0f : flex < 0f;
        }
        
        protected override float GetAcceleration(int direction, int side)
        {
            if (direction < 2) return -rbDynamics[side].angularAcceleration.x * amount[direction];
            return -rbDynamics[side].angularAcceleration.y * amount[direction];
        }

        // public override void FixedUpdate()
        // {
        //     float speed = FlexIt.flexRegressionSpeed;
        //     for (int direction = 0; direction < morphs.Count; direction++)
        //     {
        //         if(amount[direction] == 0f) continue;
        //         bool dyn = doDyn[direction];
        //         bool stat = doStatic[direction];
        //         for (int side = 0; side < morphs[direction].Count; side++)
        //         {
        //             morphs[direction][side].max = amount[direction];
        //             float mVal = morphs[direction][side].morphValue;
        //             float mTarget = 0f;
        //             bool flexed = false;
        //             float flex = 0f;
        //             if(stat)
        //             {
        //                 if((supportTrackers.Count == 0 || !supportTrackers[side].supported) && 
        //                    (children == null || !children.Any(x => x.supportTrackers[side].supported)))
        //                 {
        //                     flex = GetFlex(direction, side);
        //                     flexed = Flexed(flex, direction, side);
        //                 }
        //             }
        //             if(flexed)
        //             {
        //                 flex = Mathf.Abs(flex);
        //                 flex = ShakeIt.sin[(int)(flex * 90)];
        //                 mTarget = flex * staticStrength[direction];
        //             }
        //             if(Mathf.Abs(mVal - mTarget) > .02f)
        //             {
        //                 morphs[direction][side].morphValue = Mathf.Lerp(mVal, mTarget, speed);
        //             }
        //             if(dyn)
        //             {
        //                 
        //                 float f = GetAcceleration(direction, side);
        //                 // f.Print();
        //                 bool inRange = (direction == 1 || direction == 3) ? f > minAccel[direction] : f < -minAccel[direction];
        //                 if (inRange)
        //                 {
        //                     // f.Print();
        //                     morphBuffers[direction][side].Fill(Mathf.Abs(f) * dynamicStrength[direction] * FlexIt.realFrametime);
        //                 }
        //                 morphBuffers[direction][side].Pop();
        //             }
        //         }
        //         
        //     }
        // }
    }
}