using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Shoulder : FlexRegion
    {
        public override void Init(int id, string[] rbNames)
        {
            name = "Shoulder";
            base.Init(id, rbNames);
            
            rbDynamics = new RBDynamics[2];
            for (int i = 0; i < 2; i++)
            {
                string side = sides[i];
                var rb = FlexIt.singleton.containingAtom.forceReceivers.First(x => x.name == side.ToLower() + rbNames[0])
                    .GetComponent<Rigidbody>();
                rbDynamics[i] = rb.AddRbDynamic(i);
                SetupSupportTrackers(rb);
            }
        }
        
        protected override void SetupDefaults()
        {
            Vector4 dynamicD = dynamicStrengthJ.defaultVal;
            Vector4 minAccelD = minAccelJ.defaultVal;
            Vector4 sRatioD = sRatioJ.defaultVal;
            Vector4 amountD = amountJ.defaultVal;
            
            amountD.w = 1f;
            dynamicD = new Vector4(1.2f, 1.2f, 1.2f, 1.2f);
            
            amountJ.val = amountJ.defaultVal = amountD;
            sRatioJ.val = sRatioJ.defaultVal = sRatioD;
            dynamicStrengthJ.val = dynamicStrengthJ.defaultVal = dynamicD;
            minAccel = minAccelJ.val = minAccelJ.defaultVal = minAccelD;
            
            OnAmountChanged(amountJ.val);
            OnSRatioChanged(sRatioJ.val);
        }
        
        protected override float GetFlex(int direction, int side)
        {
            float flex;
            if (direction < 2)
            {
                flex = rbDynamics[side].transform.forward.y;
            }
            else
            {
                flex = rbDynamics[side].transform.right.y;
            }

            return flex;
        }
        
        protected override bool Flexed(float flex, int direction, int side)
        {
            bool flexed;
            if (direction < 2)
            {
                flexed = direction == 0 ? flex > 0f : flex < 0f;
            }
            else
            {
                flexed = direction == 2 ? flex > 0f : flex < 0f;
                if (side == 1) flexed = !flexed;
            }

            return flexed;
        }
        
        protected override float GetAcceleration(int direction, int side)
        {
            float val;
            if (direction < 2) val = rbDynamics[side].angularAcceleration.x;
            else val = rbDynamics[side].angularAcceleration.z;
            val *= amount[direction];
            return val;
        }
    }
}